#ifndef FINDMOTIF_H
#define FINDMOTIF_H
#include "header.h"

#include <assert.h>
#include "bindSite.h"

class FindMotif {
public:
  //FindMotif(const string trueSetFilePath, const string falseSetFilePath,
  //   const int motifLength, const double maxBindRatio);	//constructor

  FindMotif::FindMotif( const int _motif_len,
			const double _max_binding_ratio,
			vector<DNA*> &true_dna_set,
			vector<DNA*> &false_dna_set );

  ~FindMotif();		//destructor
  
  const int get_motif_len() {return motif_len;};
  Motif** get_motifArr () { return motifArr;};
  Motif* get_motif (const int i) { return motifArr[i];};
  vector<DNA*> & get_true_dna_vec() { return true_dna_vec; };
  vector<DNA*> & get_false_dna_vec() { return false_dna_vec; };
  int get_num_true_seq() { return num_true_seq; };
  int get_num_false_seq() { return num_false_seq; };
  double get_max_binding_ratio() { return max_binding_ratio;};

  static bool Get_Test_Rev() {return Test_Rev;};
  //static void Set_Test_Rev(const bool rev) {Test_Rev = rev;}
/*****************************************************************************/
/*                                                                           */
/* Pub 1 Getting data                                                        */
/*                                                                           */
/*****************************************************************************/

  //get the set of DNA regions
  //format:
  // >Name and information
  // ACGTACGTACGTACGT
  static void getDNASeq(vector<DNA*> &dnaVector, const string filePath, const int maxIn);
  void getInputDNASeq();

/*****************************************************************************/
/*                                                                           */
/* Pub 1 Algorithm                                                           */
/*                                                                           */
/*****************************************************************************/
  // Finding motif
  void discoverMotif(const int motifLength, const int noOfTurns,
		     vector<double*> &seedVec, const bool isFast);

  void FindMotif::discover_seed(const long motifLength,
				//const long maxHamDist,
				const long maxBindSite, const long outputSize,
				//Motif** motifArr,
				vector<Motif*> & motifVec,
				const bool testRev,
				const bool isFast);
  void discover_seedShort(const long motifLength, const long maxHamDist,
                          const long maxBindSite, const long outputSize,
                          //Motif** motifArr,
			  vector<Motif*> & motifVec,
			  const bool testRev);
  
  void discover_seedMed(const long motifLength, const long maxHamDist,
                        const long maxBindSite, const long outputSize,
                        //Motif** motifArr,
			vector<Motif*> & motifVec,
			const bool testRev);
  void discover_seedLong(const long motifLength, const long maxHamDist,
                         const long maxBindSite, const long outputSize,
                         //, Motif** motifArr,
			 vector<Motif*> & motifVec,
			 const bool testRev);

  void output(vector<double*> &seedVec, ostream &os);
  void output(vector<double*> &seedVec) {
    output(seedVec, cout);
  }

  // Finding seed matrix
  //void findSeed(const int noOfSeed, const int motifLength, vector<double*> &seedVec);
  
  // EM tunning for a probability Matrix
  Motif* EMLoop(double* probMat, const int numOfFused
                , const int motifLength, const int noOfTurns);

  static void Set_Num_Seeds(const int seeds) {
    Num_Seed = seeds;
  }

/*****************************************************************************/
/*                                                                           */
/* Pub 2 parameters                                                          */
/*                                                                           */
/*****************************************************************************/
  //control parameters
  const static int Max_DNA_Len=2000;			//Maximum length of a dna sequence
  const static int Max_Seed_Len = 100;
  const static int Max_Name_Len = 100;
  //Maximum length of a dna name
  const static int Num_Motif=50;			//number of output motif;
  static double Pseudo_Prob;			//min probability of a character
  
  static int Num_Seed;
  //number of seeds;

  static bool Test_Rev;
  //test reversed sequence

  const static int maxSeedLength=6;			//max seed length

  const static int seedSelectHD=1;			//Hamming Distance for selecting seeds
  const static int noOfFUsedInSeed=50;			//number of false sequences used in finding seed
  const static int maxNumOfSeq=500;			//maximum number of sequences used
  const static int seedTunedTurn=2;			//number of time tuning the seed
  
  int hamming_dist;
  void set_hamming(const int maxHamDist) {hamming_dist = maxHamDist;}

  static const long max_neigh_pat_len=8;
  static const long max_time_pat_len=12;
  static const long max_mem_pat_len=12;

  static const long minOutputCandHD=0;

  private:
/*****************************************************************************/
/*                                                                           */
/* Pri 1 parameters                                                          */
/*                                                                           */
/*****************************************************************************/
  string trueSetFilePath;
  string falseSetFilePath;
  int motif_len;
  double max_binding_ratio;
  int num_true_seq; 
  int num_false_seq;
  int avg_seq_len;
  
/*****************************************************************************/
/*                                                                           */
/* Pri 2 internal data structure                                             */
/*                                                                           */
/*****************************************************************************/
  Tools* toolsPtr;
  //Vector of DNA objects for ture set
  vector<DNA*> true_dna_vec;
  //Vector of DNA objects for false set
  vector<DNA*> false_dna_vec;
  Motif* motifArr[Num_Motif];			//Array of score for the motifs
};

#endif

