#include <fstream>
#include "getopts.h"
#include "header.h"

#include "tools.h"
#include "html.h"
#include "findMotif.h"

#define PROG_VERSION "Program version 1.07 (06/05/2005)"

/*****************************************************************************/
/*                                                                           */
/* Main function                                                             */
/*                                                                           */
/*****************************************************************************/

bool is_fast_seed=false;
uint debugLevel = 0;
string dna_true_inputfile = "";
string dna_false_inputfile = "";
unsigned int motif_len=6;
//const double maxBindRatio=2.0;
const int num_Turns=10;
const int max_seq_num = 500;
//const int Max_Seed_Len = 6;
int max_binding = 2;
vector<DNA*> trueSetDNAVec;
vector<DNA*> falseSetDNAVec;

void output_plain ( ostream &os = cout );
void output_HTML ( ostream &os, const int time_taken=0 );

ofstream outputf;
int output_size = 50;

long maxHamDist = -1; // motif_len/4 - 1
long maxBindSite=5;
long outputSize=50;

void getDNASeq(vector<DNA*> &dnaVector,
	       const char *filePath,
	       const int maxIn);

string output_file = "./output.html";


int main(int argc, char *argv[]) {
  char *args;
  int c;
  struct options opts[] =
    {
      //{ 1, "debug",   "Set the debugging level", "d", 1 },
      //{ 2, "version", "Display the version info", "V", 0 },
      { 1, "binding",   "Max binding sites per sequence (default 2)", "b", 1 },
      { 2, "seeds",     "Number of seeds used (default 100)", "s", 1 },
      //{ 3, "debug",   "Set the debugging level", "d", 1 },
      { 3, "true",    "True file", "t", 1 },
      { 4, "false",   "False file", "f", 1 },
      { 5, "motif",   "Motif length", "m", 1 },
      { 6, "no_reverse", "do not test reversed sequences", "r", 0},
      { 7, "outputsize", "number of motifs to output (50)", "o", 1},
      { 8, "outputfile", "output file name (html)", "O", 1 },
      { 9, "fast_seed", "speed up the program", "v", 0},
      //      { 4, "test",    "Test getopts_usage",       "t", 0 },
      { 0, NULL,      NULL,                      NULL, 0 }
    };
  
  while ((c = getopts(argc, argv, opts, &args)) != 0) {
    int level = -1;
    
    switch(c) {
    case 1:
      level = atoi(args);
      if ( level >= 10 || level <= 0) {
	cerr << "maximum binding sites cannot be between 1 and 9." << endl;
	exit(0);
      }

      max_binding = level;
      break;
    case 2:
      level = atoi(args);
      if ( level >= 50000 || level <= 5) {
	cerr << "number of seeds used shoud be between 5 and 50,000." << endl;
	exit(0);
      }
      FindMotif::Set_Num_Seeds(level);
      //max_binding = level;
      //printf("ALSE 0.1.0\n");
      //printf("Written by: Henry Chi Ming Leung, Packaged by Ho Ngai Lam\n");
      //exit(0);
      break;
    case 3:
      // printf("True input file: %s\n", args);
      dna_true_inputfile = args;
      break;
    case 4:
      // printf("False input file: %s\n", args);
      dna_false_inputfile = args;
      break;
    case 5:
      level = atoi(args);
      if ( level <=2 ) {
	cerr << "Motif length cannot be too short." << endl;
      } else {
	motif_len = level;
      }
      break;
    case 6:
      FindMotif::Test_Rev = false;
      break;
    case 7:
      level = atoi(args);
      if ( level < 1 || level >= 1000) {
	fprintf(stderr,"Error in output size: enter a number between 1 to  1000\n");
	exit(-1);
      }
      output_size = (level);
      //assert (0);
      break;
    case 8:
      output_file = (args);
      break;
    case 9:
      is_fast_seed = true;
      break;
    case -2:
      printf("Unknown Option: %s\n", args);
      break;
      /* Special Case: getopts() can't allocate memory.
	 Should probably exit() here */
    case -1:
      printf("Unabled to allocate memory from getopts().\n");
      break;
    default:
      break;
    }
      /* This free() is required since getopts() automagically allocates space
	 for "args" everytime it's called. */
      free(args);
    }

  //printf ("dna_true_inputfile %d\n", dna_true_inputfile.length() );
  //printf ("dna_false_inputfile %d\n", dna_false_inputfile.length() );

  if ( dna_true_inputfile.length()==0 ||
       dna_false_inputfile.length()==0 ) {
    cerr << "\n\tError:Please specify the input files (options -t and -f)\n" << endl;
    exit(-1);
  }

  getDNASeq(trueSetDNAVec, dna_true_inputfile.c_str(), max_seq_num);
  getDNASeq(falseSetDNAVec, dna_false_inputfile.c_str(), max_seq_num);
  
  // random seed
  srand(time(0));
#ifndef NDEBUG
  //cout << "Debug Level=\t"     << debugLevel << endl;
#endif
 
  output_plain( );



  long start=time(0);
  //long current=time(0);
  
  cout << PROG_VERSION <<endl;

  FindMotif fm (motif_len, (double)max_binding,
		trueSetDNAVec, falseSetDNAVec );

  time_t now = time(NULL);
  printf("\n\tstart finding motifs... --> %s\n", ctime(&now) );

  vector<double *> seedVec;
  if ( maxHamDist < 0 ) {
    // uninitiatised
    maxHamDist = max( 0, (int)motif_len/4 -1);
  }
  //printf ("setting hamming dist => %d\n", maxHamDist);
  fm.set_hamming(maxHamDist);
  //FindMotif::Set_Test_Rev();

  fm.discoverMotif(motif_len, num_Turns, seedVec, is_fast_seed);

  time_t time_taken = time(NULL)-start;

  fm.output(seedVec);

  printf("\n\tTotal Time taken = %dm %ds\n\n\a\n",
	 (int)(time_taken)/60, (int)(time_taken)%60 );  
  
  outputf.open(output_file.c_str());
  if (outputf) {
    output_HTML(outputf, time_taken);
    HTML_output html(fm,output_size);
    html.output(seedVec,outputf);
  }


  //fm.output_HTML(seedVec,outputf);

  return 0;

}

void getDNASeq(vector<DNA*> &dnaVector,
	       const char *filename,
	       const int maxIn) {
  ifstream seq_file(filename);
  if (!seq_file) {
    cout << "Sequence file not found: " << filename;
    exit(-999);
  }
 
#define LENGTH 65535
  char line[LENGTH];
  string tmp_seq("");
  char seq_name[LENGTH] = "";

  DNA *dna = NULL;
 
  while (!seq_file.eof()) {
    seq_file.getline(line,LENGTH);
    if (line[0] == '>') {
      //new sequene
      int seq_length = tmp_seq.length();
      if ( seq_length > 0 ) {
	//Sequence seq(seq_name, tmp_seq.c_str() );
	//sequences.push_back(seq);
	//tmp_seq = "";
	dna = new DNA( strdup(seq_name), strdup(tmp_seq.c_str()) );
	dnaVector.push_back(dna);
	tmp_seq = "";
      }
      strncpy(seq_name, line + 1, LENGTH-1);
    } else {
      char *end = line + LENGTH;
      char *current = line;
      while (current < end) {
	if ( *current == 0 ) break;
	char c = *current;
	if ( c == '#' ) break; // ignore comment
	c = toupper(c);
	if ( 'A' <= c && c <= 'Z' ) {
	  tmp_seq += c;
	}
	current ++;
      }
    }
  }

  // last sequence
  dna = new DNA( strdup(seq_name), strdup(tmp_seq.c_str()) );
  dnaVector.push_back(dna);
  tmp_seq = "";
  //quence seq(seq_name, tmp_seq.c_str() );
  //quences.push_back(seq);
  //q_file.close();

  seq_file.close();
}


void output_plain ( ostream &os ) {
  os << endl << "True set file:\t"    << dna_true_inputfile << endl;
  os << "False set file:\t"   << dna_false_inputfile << endl;
  os << "Motif length:\t"     << motif_len << endl;
  //os << "max bind ratio:\t"   << max_binding << endl;
  //os << "num of turns:\t"     << num_Turns << endl;
  os << "num of seeds:\t"    << FindMotif::Num_Seed << endl;
  //os << "pseudo prob.:\t"    << FindMotif::Pseudo_Prob << endl;
  os << "test reversed:\t"   << (FindMotif::Test_Rev?"True":"False") << endl;
  //os << "max length of seed:\t"   << FindMotif::Max_Seed_Len << endl;
  //os << "select seed hamming distance:\t"    << FindMotif::seedSelectHD << endl;
  //os << "num_FUsedInSeed=\t" << FindMotif::num_FUsedInSeed << endl;
  //os << "max_seq_num:\t"     << max_seq_num << endl;
  //os << "seedTunedTurn=\t"   << FindMotif::seedTunedTurn << endl << endl;

  os << endl << "output HTML filename:\t" << output_file << endl;
}

void output_HTML ( ostream &os, const int time_taken )  {
    outputf << "<html>\n  <head><title>&nbsp;&nbsp;&nbsp;ALSE output result</title>";
    outputf << "<style><!--" << endl
	    << "body,td,div,.p,a{font-family:arial,sans-serif}" << endl
	    << "h1{color:#000088;font-family:times,times;font-size:20pt;background-color:#99FFCC;line-height:38pt;vertical-align:text-bottom;text-indent:10pt}" << endl
	    << "h2{color:#000088;font-size:16pt;background-color:#99FFCC;line-height:30pt;text-indent:10pt}" << endl
	    << "h3{color:#112299;font-size:14pt;font-style:bold;line-height:25pt;text-indent:2pt}" << endl
	    << "h4{color:#112299;font-size:12pt;font-style:italic;line-height:20pt;text-indent:1pt}" << endl
	    << "a{text-decoration:none}" << endl
	    << "a:hover{text-decoration:underline;background-color:#8EE5EE;}" << endl
	    << "a:visit{color:#0000ff}" << endl 
	    << ".headerrow{font-family:times,MS Serif,Chicago;font-size:14pt;background-color:#dddddd;color:#444444;font-weight:bold;text-align:center;}" << endl
	    << ".oddrow{font-size:12pt;background-color:#f0fff0;vertical-align:text-top}" << endl
	    << ".evenrow{font-size:12pt;background-color:#ddffff;vertical-align:text-top}" << endl
	    << ".style_A{font-weight:bold;font-family:monospace;color:#006400}"<<endl
	    << ".style_C{font-weight:bold;font-family:monospace;color:#00008B}"<<endl
	    << ".style_G{font-weight:bold;font-family:monospace;color:#FF8000}"<<endl
	    << ".style_T{font-weight:bold;font-family:monospace;color:#006400}"<<endl
	    << ".style__{font-weight:normal;font-family:monospace;color:#606060}"<<endl
	    << "table{border:1;border-style:none;padding:1;padding-left:2;background-color:darkgrey}" << endl
	    << "  //--></style>" << endl 
	    << "  </head>\n  <body>" << endl;

    const time_t now = time(NULL);

    outputf << "<h1> ALSE - Motif Finding Tool </h1>" << endl
	    << "Department of Computer Science, The University of Hong Kong &copy;<br/>" << PROG_VERSION << "<br/>" << ctime(&now) << "<hr/>\n" << endl;
    outputf << "<h2>Program Parameters</h2>" << endl;

    outputf << "    <table>" << endl;
    outputf << "<tr class='oddrow'><td width='30%'>True set file:</td><td width='70%'><tt>"    

	    << dna_true_inputfile << "</tt><p/><i><font size='-1'><ul>";
    int j=0;
    for (vector<DNA*>::iterator i = trueSetDNAVec.begin();
	 i != trueSetDNAVec.end();
	 i++) {
      if (j++ >= 10) {
	outputf << "  <li>etc...</li>" << endl;
	break;
      }
      outputf << "  <li>" << (*i)->getName() << "</li>" << endl;
    }
    outputf << "</ul></font></i></td></tr>" << endl;

    outputf << "<tr class='evenrow'><td>False set file (background sequence): </td><td><tt>" 
	    << dna_false_inputfile << "</tt><p/><i><font size='-1'><ul>";
    j = 0;
    for (vector<DNA*>::iterator i = falseSetDNAVec.begin();
	 i != falseSetDNAVec.end();
	 i++) {
      if (j++ >= 10) {
	outputf << "  <li>etc...</li>" << endl;
	break;
      }
      outputf << "<li>" << (*i)->getName() << "</li>" << endl;
    }
    outputf << "</font></ul></i></td></tr>" << endl;

    outputf << "<tr class='oddrow'><td>motif length: </td><td>"   
	    << motif_len << "</td></tr>" << endl;
    //outputf << "<tr class='evenrow'><td>max. no. motifs in sequence: </td><td>" 
    //<<  max_binding << "</td></tr>" << endl;
    //outputf << "<tr class='oddrow'><td>num of tuning iterations: </td><td>" << num_Turns << "</td></tr>" << endl;
    outputf << "<tr class='evenrow'><td>no. of seeds used: </td><td>"   
	    << FindMotif::Num_Seed << "</td></tr>" << endl;
    outputf << "<tr class='evenrow'><td>test reversed sequences: </td><td>"  
	    << (FindMotif::Test_Rev?"True":"False") << "</td></tr>" << endl;
    outputf << "<tr class='oddrow'><td>no. of seeds used: </td><td>" << FindMotif::Max_Seed_Len << "</td></tr>" << endl;

    outputf << "<tr class='evenrow'><td>output HTML filename: </td><td><tt>"
	    << output_file << "</tt></td></tr>" << endl;

    if (time_taken > 0) {
      outputf << "<tr class='oddrow'><td>execution time: </td><td><tt>"
	      << (time_taken/60) << " mins(s) " << (time_taken%60) << " secs "
	      << "</tt></td></tr>" << endl;
    }

    outputf << "    </table><hr/>" << endl;
    // motifs information to follow up
}

