#include "header.h"

/*****************************************************************************/
/*                                                                           */
/* Constructor and Destructor                                                */
/*                                                                           */
/*****************************************************************************/
Motif::Motif(double* probMat, const double score, const int* bsDistributeT
        , const int* bsDistributeF, const double alpha) {
  this->probMat=probMat;
  this->score=score;
  this->bsDistributeT=bsDistributeT;
  this->bsDistributeF=bsDistributeF;
  this->alpha=alpha;
}

Motif::~Motif() {
  if(probMat!=NULL)
    delete [] probMat;
  if(bsDistributeT!=NULL)
    delete [] bsDistributeT;
  if(bsDistributeF!=NULL)
    delete [] bsDistributeF;
}

/*****************************************************************************/
/*                                                                           */
/* Pub 1 Access Attributes                                                   */
/*                                                                           */
/*****************************************************************************/

//get the probability matrix of the motif
const double* Motif::getProbMat(){
  return probMat;
}

//get the score of the motif
double Motif::getScore() {
  return score;
}

//get the distribution of the binding sites in true set of the motif
const int* Motif::getBSDistributeT(){
  return bsDistributeT;
}

//get the distribution of the binding sites in false set of the motif
const int* Motif::getBSDistributeF(){
  return bsDistributeF;
}

//get the threshold alpha of the motif
double Motif::getAlpha() {
  return alpha;
}

int Motif::sortFunc( const void *a, const void *b)
{
  Motif* motif1=*((Motif**)a);
  Motif* motif2=*((Motif**)b);
   
  if(motif1==NULL)
    return -1;
     
  if(motif2==NULL)
    return 1;
    
  if(motif1->getScore() < motif2->getScore())
    return -1;
  if(motif1->getScore() == motif2->getScore())
    return 0;
  return 1;
 
}
