#ifndef DNA_H
#define DNA_H
#include "header.h"
#include "bindSite.h"

class DNA {
public:
  DNA(const char* name, const char* sequence);	//constructor
  ~DNA();				//destructor
  
/*****************************************************************************/
/*                                                                           */
/* Pub 1 Access Attributes                                                   */
/*                                                                           */
/*****************************************************************************/

  //get the name of the dna
  const char *getName();
  const char *getSeq();
  const char *getSeqR();
  const double *getScoreArr();
  const double *getScoreArrR();
  
/*****************************************************************************/
/*                                                                           */
/* Pub 2 Algorithms                                                          */
/*                                                                           */
/*****************************************************************************/
  
  //Calculate score of all substrings
  void calculateScore(const double* probMat, const int motifLength, const bool testRer);
  
  //Find if a pattern exist in the dna sequence with Hamming distance <= distance
  int findSubstring(const char* pattern, const int distance, const int stringLength
                    , const bool testRev);


  void findBindSite(const char* pattern, const long hamDist,
		    vector<BindSite*> &bindSiteVec, const bool testRev);
  
  short findBindSiteNum(const char* pattern, const long hamDist,
			const bool testRev);

/*****************************************************************************/
/*                                                                           */
/* Pub 3 Parameters                                                          */
/*                                                                           */
/*****************************************************************************/  
  //static const double dependPara=0;


  
  private:
/*****************************************************************************/
/*                                                                           */
/* Pri 1 Algorithms                                                          */
/*                                                                           */
/*****************************************************************************/
  
  //Calculation log probability of a substring generated from a matrix probMat
  static double calLogProbMat(const double* probMat, const char* bindingSite, const int motifLength);

  //Calculation log probability of a substring generated from a background probability
  static double calLogBackProb(const double* backgroundMat, const char* bindingSite, const int motifLength);
  
/*****************************************************************************/
/*                                                                           */
/* Pri 2 Parameters                                                          */
/*                                                                           */
/*****************************************************************************/
  const char* dnaName;
  const char* dnaSeq;
  const char* dnaSeqR;
  double* scoreArr;
  double* scoreArrR;
};

#endif

